@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: rulflo.scm,v 1.29 2007/01/05 21:19:21 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -264,8 +262,9 @@ USA.
 			   (FSTP (ST ,',(1+ target)))))))))))))
   (define-flonum-operation FLONUM-NEGATE FCHS)
   (define-flonum-operation FLONUM-ABS FABS)
-  (define-flonum-operation FLONUM-SIN FSIN)
-  (define-flonum-operation FLONUM-COS FCOS)
+  ;; Disabled: FSIN and FCOS limited to pi * 2^62.
+  ;;(define-flonum-operation FLONUM-SIN FSIN)
+  ;;(define-flonum-operation FLONUM-COS FCOS)
   (define-flonum-operation FLONUM-SQRT FSQRT)
   (define-flonum-operation FLONUM-ROUND FRNDINT))
 
@@ -316,12 +315,6 @@ USA.
 (define-arithmetic-method 'FLONUM-LOG flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
-     #|
-     (LAP (FLDLN2)
-	  (FLD (ST ,(1+ source)))
-	  (FYL2X)
-	  (FSTP (ST ,(1+ target))))
-     |#
      (LAP (FLDLN2)
 	  (FXCH (ST 0) (ST 1))
 	  (FYL2X)))))
@@ -329,43 +322,57 @@ USA.
 (define-arithmetic-method 'FLONUM-EXP flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
-     #|
-     (LAP (FLD (ST ,source))
-	  (FLDL2E)
-	  (FMULP (ST 1) (ST 0))
-	  (F2XM1)
-	  (FLD1)
-	  (FADDP (ST 1) (ST 0))
-	  (FSTP (ST ,(1+ target))))
-     |#
-     (LAP (FLDL2E)
-	  (FMULP (ST 1) (ST 0))
-	  (F2XM1)
-	  (FLD1)
-	  (FADDP (ST 1) (ST 0))))))
-
+     ;; Hair to avoid arithmetic for non-finite inputs: exp(-inf) = 0,
+     ;; but exp(x) = x for any other non-finite x.  We use the first
+     ;; free slot (1) to pick apart the double format to check for
+     ;; non-finite inputs, and (2) to avoid using two stack slots.
+     (let ((temp (temporary-register-reference))
+	   (infinity-or-nan (generate-label 'INFINITY-OR-NAN))
+	   (join (generate-label 'JOIN))
+	   (temp-pointer regnum:free-pointer))
+       (LAP (FST D (@R ,temp-pointer))
+	    (MOV W ,temp (@RO W ,temp-pointer 4))
+	    (AND W ,temp (&U #x7FFFFFFF))
+	    (CMP W ,temp (&U #x7FF00000))
+	    (JAE B (@PCR ,infinity-or-nan))
+	    ;; Compute 2^(x log_2 e) with F2XM1 and FSCALE.
+	    (FLDL2E)			;st0 = lg e, st1 = x
+	    (FMULP (ST 1) (ST 0))	;st0 = x lg e
+	    (FLD (ST 0))		;st0 = x lg e, st1 = x lg e
+	    (FRNDINT)			;st0 = I(x lg e), st1 = x lg e
+	    (FSUB (ST 1) (ST 0))	;st0 = I(x lg e), st1 = F(x lg e)
+	    (FSTP D (@R ,temp-pointer)) ;st0 = F(x lg e), save I(x lg e)
+	    (F2XM1)			;st0 = 2^F(x lg e) - 1
+	    (FLD1)			;st0 = 1, st1 = 2^F(x lg e) - 1
+	    (FADD)			;st0 = 2^F(x lg e)
+	    (FLD D (@R ,temp-pointer))	;st0 = I(x lg e), st1 = 2^F(x lg e)
+	    (FXCH (ST 0) (ST 1))	;st0 = 2^F(x lg e), st1 = I(x lg e)
+	    (FSCALE)			;st0 = 2^F(x lg e) * 2^I(x lg e),
+					;st1 = I(x lg e)
+	    (FSTP (ST 1))		;Drop st1, leaving in st0 the value
+	    (JMP B (@PCR ,join))	; 2^(F(x lg e) + I(x lg e)) = e^x.
+	  (LABEL ,infinity-or-nan)
+	    (CMP W (@RO W ,temp-pointer 4) (&U #xFFF00000))
+	    (JNE B (@PCR ,join))
+	    (CMP W (@RO W ,temp-pointer 0) (& 0))
+	    (JNE B (@PCR ,join))
+	    (FSTP (ST 0))		;Pop argument.
+	    (FLDZ)			;Return zero.
+	  (LABEL ,join))))))
+
+#|
+;; Disabled: FPTAN limited to pi * 2^62.
 (define-arithmetic-method 'FLONUM-TAN flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
-     #|
-     (LAP (FLD (ST ,source))
-	  (FPTAN)
-	  (FSTP (ST 0))			; FPOP
-	  (FSTP (ST ,(1+ target))))
-     |#
      (LAP (FPTAN)
 	  (FSTP (ST 0))			; FPOP
 	  ))))
-
+|#
+
 (define-arithmetic-method 'FLONUM-ATAN flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
-     #|
-     (LAP (FLD (ST ,source))
-	  (FLD1)
-	  (FPATAN)
-	  (FSTP (ST ,(1+ target))))
-     |#
      (LAP (FLD1)
 	  (FPATAN)))))
 
@@ -375,16 +382,6 @@ USA.
 (define-arithmetic-method 'FLONUM-ACOS flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
-     #|
-     (LAP (FLD (ST ,source))
-	  (FMUL (ST 0) (ST 0))
-	  (FLD1)
-	  (F%SUBP (ST 1) (ST 0))
-	  (FSQRT)
-	  (FLD (ST ,(1+ source)))
-	  (FPATAN)
-	  (FSTP (ST ,(1+ target))))
-     |#
      (LAP (FST D (@R ,regnum:free-pointer))
 	  (FMUL (ST 0) (ST 0))
 	  (FLD1)
@@ -396,17 +393,6 @@ USA.
 (define-arithmetic-method 'FLONUM-ASIN flonum-methods/1-arg
   (flonum-unary-operation/stack-top
    (lambda ()
-     #|
-     (LAP (FLD (ST ,source))
-	  (FMUL (ST 0) (ST 0))
-	  (FLD1)
-	  (F%SUBP (ST 1) (ST 0))
-	  (FSQRT)
-	  (FLD (ST ,(1+ source)))
-	  (FXCH (ST 0) (ST 1))
-	  (FPATAN)
-	  (FSTP (ST ,(1+ target))))
-     |#
      (LAP (FST D (@R ,regnum:free-pointer))
 	  (FMUL (ST 0) (ST 0))
 	  (FLD1)
