@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: buffer.scm,v 1.192 2005/03/31 19:00:03 cph Exp $
-
-Copyright 1986,1989,1990,1991,1992,1993 Massachusetts Institute of Technology
-Copyright 1994,1995,1996,1998,1999,2000 Massachusetts Institute of Technology
-Copyright 2001,2002,2003,2005 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -20,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -175,6 +173,9 @@ The buffer is guaranteed to be deselected at that time."
 (define-integrable (%set-buffer-point! buffer mark)
   (set-group-point! (buffer-group buffer) mark))
 
+(define-integrable (%set-buffer-point-index! buffer index)
+  (set-group-point-index! (buffer-group buffer) index))
+
 (define-integrable (minibuffer? buffer)
   (char=? (string-ref (buffer-name buffer) 0) #\Space))
 
@@ -243,12 +244,12 @@ The buffer is guaranteed to be deselected at that time."
   (set-buffer-alist! buffer (del-assq! key (buffer-alist buffer))))
 
 (define (->buffer object)
-  (or (cond ((buffer? object) object)
+  (or (cond ((or (default-object? object) (not object)) (current-buffer))
+	    ((buffer? object) object)
 	    ((mark? object) (mark-buffer object))
 	    ((group? object) (group-buffer object))
 	    ((region? object) (mark-buffer (region-start object)))
 	    ((window? object) (window-buffer object))
-	    ((not object) (current-buffer))
 	    (else (error:wrong-type-argument object "buffer" '->BUFFER)))
       (error:bad-range-argument object '->BUFFER)))
 
