@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: editor.scm,v 1.264 2007/01/05 21:19:23 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -89,7 +87,8 @@ USA.
 	      message))))))))
 
 (define (edwin . args) (apply edit args))
-(simple-command-line-parser "edit" edit)
+(simple-command-line-parser "edit" edit
+			    "Causes Edwin to start immediately after Scheme.")
 
 (define edwin-editor #f)
 (define editor-abort)
@@ -600,10 +599,12 @@ TRANSCRIPT    messages appear in transcript buffer, if it is enabled;
 (define (inferior-thread-output! flags)
   (without-interrupts (lambda () (inferior-thread-output!/unsafe flags))))
 
-(define-integrable (inferior-thread-output!/unsafe flags)
+(define (inferior-thread-output!/unsafe flags)
   (set-car! flags #t)
-  (set! inferior-thread-changes? #t)
-  (signal-thread-event editor-thread #f))
+  (if (not inferior-thread-changes?)
+      (begin
+	(set! inferior-thread-changes? #t)
+	(signal-thread-event editor-thread #f))))
 
 (define (accept-thread-output)
   (with-interrupt-mask interrupt-mask/gc-ok
