@@ -1,8 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: filcom.scm,v 1.226 2003/02/14 18:28:12 cph Exp $
-
-Copyright 1986, 1989-2001 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -242,12 +242,7 @@ procedures are called."
 					       car
 					       name)))))))
       (if entry
-	  (begin
-	    (local-set-variable! scheme-environment (cadr entry) buffer)
-	    (if (and (eq? 'DEFAULT (ref-variable scheme-environment buffer))
-		     (not (eq? 'DEFAULT (cadr entry))))
-		(message "Ignoring bad evaluation environment: "
-			 (cadr entry))))))))
+	  (local-set-variable! scheme-environment (cadr entry) buffer)))))
 
 (define (find-file-revert buffer)
   (if (verify-visited-file-modification-time? buffer)
