@@ -1,23 +1,26 @@
 #| -*-Scheme-*-
 
-$Id: tterm.scm,v 1.34 2002/02/09 05:55:25 cph Exp $
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
-Copyright (c) 1990-1999, 2001, 2002 Massachusetts Institute of Technology
+This file is part of MIT/GNU Scheme.
 
-This program is free software; you can redistribute it and/or modify
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.
 
-This program is distributed in the hope that it will be useful, but
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
 |#
 
 ;;;; Termcap(3) Screen Implementation
@@ -92,15 +95,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 	 (channel-type=terminal? channel)
 	 (terminal-output-baud-rate channel))))
 
-(define (output-port/buffered-chars port)
-  (let ((operation (output-port/operation port 'BUFFERED-CHARS)))
+(define (output-port/buffered-bytes port)
+  (let ((operation (port/operation port 'BUFFERED-OUTPUT-BYTES)))
     (if operation
 	(operation port)
 	0)))
 
-(define (output-port/y-size port)
-  ((output-port/operation port 'Y-SIZE) port))
-
 (define (console-available?)
   (let ((description (console-termcap-description)))
     (and (termcap-description? description)
@@ -197,7 +197,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 				 (let ((code (vector-8b-ref string start)))
 				   (if (fix:< code #x80)
 				       (make-char code 0)
-				       (make-char (fix:and code #x7f) 1))))))
+				       (make-char (fix:and code #x7F)
+						  char-bit:meta))))))
 			 (let* ((key-seq  (caar key-pairs))
 				(n-seq    (string-length key-seq)))
 			   (cond ((and (fix:<= n-seq n-chars)
@@ -247,9 +248,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 			((not have-select?)
 			 (and block? (read-event block?)))
 			(else
-			 (case (test-for-input-on-descriptor
-				(channel-descriptor-for-select channel)
-				block?)
+			 (case (test-for-io-on-channel channel 'READ block?)
 			   ((#F) #f)
 			   ((PROCESS-STATUS-CHANGE) event:process-status)
 			   ((INTERRUPT) (loop))
@@ -441,46 +440,48 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   (scroll-region false)
   (key-table false))
 
-(let-syntax ((define-accessor
-	      (sc-macro-transformer
-	       (lambda (form environment)
-		 (let ((name (cadr form)))
-		   `(DEFINE-INTEGRABLE (,(symbol-append 'SCREEN- name) SCREEN)
-		      (,(close-syntax (symbol-append 'TERMINAL-STATE/ name)
-				      environment)
-		       (SCREEN-STATE SCREEN)))))))
-	     (define-updater
-	      (sc-macro-transformer
-	       (lambda (form environment)
-		 (let ((name (cadr form)))
-		   (let ((param (make-synthetic-identifier name)))
-		     `(DEFINE-INTEGRABLE
-			(,(symbol-append 'SET-SCREEN- name '!) SCREEN ,param)
-			(,(close-syntax
-			   (symbol-append 'SET-TERMINAL-STATE/ name '!)
-			   environment)
-			 (SCREEN-STATE SCREEN)
-			 ,param))))))))
-  (define-accessor description)
-  (define-accessor baud-rate-index)
-  (define-accessor baud-rate)
-  (define-accessor insert-line-cost)
-  (define-accessor insert-line-next-cost)
-  (define-accessor delete-line-cost)
-  (define-accessor delete-line-next-cost)
-  (define-accessor scroll-region-cost)
-  (define-accessor cursor-x)
-  (define-updater  cursor-x)
-  (define-accessor cursor-y)
-  (define-updater  cursor-y)
-  (define-accessor standout-mode?)
-  (define-updater  standout-mode?)
-  (define-accessor insert-mode?)
-  (define-updater  insert-mode?)
-  (define-accessor delete-mode?)
-  (define-updater  delete-mode?)
-  (define-accessor scroll-region)
-  (define-updater  scroll-region))
+(define-syntax define-ts-accessor
+  (sc-macro-transformer
+   (lambda (form environment)
+     (let ((name (cadr form)))
+       `(DEFINE-INTEGRABLE (,(symbol-append 'SCREEN- name) SCREEN)
+	  (,(close-syntax (symbol-append 'TERMINAL-STATE/ name)
+			  environment)
+	   (SCREEN-STATE SCREEN)))))))
+
+(define-syntax define-ts-modifier
+  (sc-macro-transformer
+   (lambda (form environment)
+     (let ((name (cadr form)))
+       (let ((param (make-synthetic-identifier name)))
+	 `(DEFINE-INTEGRABLE
+	    (,(symbol-append 'SET-SCREEN- name '!) SCREEN ,param)
+	    (,(close-syntax
+	       (symbol-append 'SET-TERMINAL-STATE/ name '!)
+	       environment)
+	     (SCREEN-STATE SCREEN)
+	     ,param)))))))
+
+(define-ts-accessor description)
+(define-ts-accessor baud-rate-index)
+(define-ts-accessor baud-rate)
+(define-ts-accessor insert-line-cost)
+(define-ts-accessor insert-line-next-cost)
+(define-ts-accessor delete-line-cost)
+(define-ts-accessor delete-line-next-cost)
+(define-ts-accessor scroll-region-cost)
+(define-ts-accessor cursor-x)
+(define-ts-modifier cursor-x)
+(define-ts-accessor cursor-y)
+(define-ts-modifier cursor-y)
+(define-ts-accessor standout-mode?)
+(define-ts-modifier standout-mode?)
+(define-ts-accessor insert-mode?)
+(define-ts-modifier insert-mode?)
+(define-ts-accessor delete-mode?)
+(define-ts-modifier delete-mode?)
+(define-ts-accessor scroll-region)
+(define-ts-modifier scroll-region)
 
 ;;;; Console Screen Operations
 
@@ -490,11 +491,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   unspecific)
 
 (define (console-enter! screen)
+  (add-event-receiver! event:console-resize resize-screen)
   (maybe-output screen (ts-enter-termcap-mode (screen-description screen)))
   (set-screen-cursor-x! screen false)
   (set-screen-cursor-y! screen false))
 
 (define (console-exit! screen)
+  (remove-event-receiver! event:console-resize resize-screen)
   (let ((description (screen-description screen)))
     (move-cursor screen 0 (fix:-1+ (screen-y-size screen)))
     (exit-standout-mode screen)
@@ -509,11 +512,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 (define (console-wrap-update! screen thunk)
   screen
   (let ((finished? (thunk)))
+    (window-direct-output-cursor! (screen-cursor-window screen))
     (output-port/flush-output console-i/o-port)
     finished?))
 
 (define (console-discretionary-flush screen)
-  (let ((n (output-port/buffered-chars console-i/o-port)))
+  (let ((n (output-port/buffered-bytes console-i/o-port)))
     (if (fix:< 20 n)
 	(begin
 	  (output-port/flush-output console-i/o-port)
@@ -1034,7 +1038,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 		     (if scroll-region
 			 (cdr scroll-region)
 			 (tn-y-size (screen-description screen))))
-		   (screen-cursor-y screen))))
+		   (or (screen-cursor-y screen) 0))))
 
 (define-integrable (output-1 screen command)
   (output-n screen command 1))
