@@ -1,23 +1,27 @@
-;;; -*-Scheme-*-
-;;;
-;;; $Id: imap-response.scm,v 1.44 2001/10/10 04:26:43 cph Exp $
-;;;
-;;; Copyright (c) 2000-2001 Massachusetts Institute of Technology
-;;;
-;;; This program is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU General Public License as
-;;; published by the Free Software Foundation; either version 2 of the
-;;; License, or (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;;; 02111-1307, USA.
+#| -*-Scheme-*-
+
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+
+This file is part of MIT/GNU Scheme.
+
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
+|#
 
 ;;;; IMAP Server Response Reader
 
@@ -26,7 +30,7 @@
 (define (imap:read-server-response port)
   (let ((tag (read-string-internal char-set:space port)))
     (if (eof-object? tag)
-	tag
+	(error "IMAP server unexpectedly disconnected.")
 	(let ((response
 	       (if (string=? "+" tag)
 		   (cons 'CONTINUE (read-response-text port))
@@ -75,7 +79,10 @@
 
 (define (read-flags-response port)
   (discard-known-char #\space port)
-  (read-list port read-flag))
+  ;; Work around bug in Gmail IMAP server: FLAGS response improperly
+  ;; contains the \* flag.
+  ;;(read-list port read-flag)
+  (delq! '|\\*| (read-list port read-pflag)))
 
 (define (read-list-response port)
   (discard-known-char #\space port)
@@ -260,9 +267,9 @@
 	  (else (error "Illegal astring syntax:" char)))))
 
 (define (read-nstring input)
-  (let ((output (make-accumulator-output-port)))
+  (let ((output (open-output-string)))
     (and (read-nstring-to-port input output)
-	 (get-output-from-accumulator output))))
+	 (get-output-string output))))
 
 (define (read-nstring-to-port input output)
   (let ((char (peek-char-no-eof input)))
@@ -280,9 +287,9 @@
 	  (else (error "Illegal astring syntax:" char)))))
 
 (define (read-quoted input)
-  (with-string-output-port
-    (lambda (output)
-      (read-quoted-to-port input output))))
+  (call-with-output-string
+   (lambda (output)
+     (read-quoted-to-port input output))))
 
 (define (read-quoted-to-port input output)
   (discard-known-char #\" input)
@@ -293,7 +300,7 @@
 	       (write-char char output)
 	       (loop))
 	      ((char=? #\\ char)
-	       (let ((char (read-char-no-eof char)))
+	       (let ((char (read-char-no-eof input)))
 		 (if (imap:quoted-special? char)
 		     (begin
 		       (write-char char output)
@@ -303,9 +310,9 @@
 	       (lose)))))))
 
 (define (read-literal input)
-  (with-string-output-port
-    (lambda (output)
-      (read-literal-to-port input output))))
+  (call-with-output-string
+   (lambda (output)
+     (read-literal-to-port input output))))
 
 (define (read-literal-to-port input output)
   (discard-known-char #\{ input)
@@ -529,13 +536,15 @@
 
 (define (read-char-internal port)
   (let ((char (read-char port)))
-    (if imap-transcript-port
+    (if (and (not (eof-object? char))
+             imap-transcript-port)
 	(write-char char imap-transcript-port))
     char))
 
 (define (read-string-internal delimiters port)
   (let ((s (read-string delimiters port)))
-    (if imap-transcript-port
+    (if (and (not (eof-object? s))
+             imap-transcript-port)
 	(write-string s imap-transcript-port))
     s))
 
