@@ -1,8 +1,8 @@
 /* -*-C-*-
 
-$Id: osctty.h,v 1.6 2003/02/14 18:28:22 cph Exp $
-
-Copyright (c) 1990-1999 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 */
@@ -30,16 +30,16 @@ USA.
 
 /* If this procedure returns 0, the interrupt control procedures will
    not work correctly. */
-extern int EXFUN (OS_ctty_interrupt_control, (void));
+extern int OS_ctty_interrupt_control (void);
 
 typedef unsigned int Tinterrupt_enables;
-extern void EXFUN (OS_ctty_get_interrupt_enables, (Tinterrupt_enables * mask));
-extern void EXFUN (OS_ctty_set_interrupt_enables, (Tinterrupt_enables * mask));
+extern void OS_ctty_get_interrupt_enables (Tinterrupt_enables * mask);
+extern void OS_ctty_set_interrupt_enables (Tinterrupt_enables * mask);
 
-extern unsigned int EXFUN (OS_ctty_num_int_chars, (void));
-extern cc_t * EXFUN (OS_ctty_get_int_chars, (void));
-extern cc_t * EXFUN (OS_ctty_get_int_char_handlers, (void));
-extern void EXFUN (OS_ctty_set_int_chars, (cc_t *));
-extern void EXFUN (OS_ctty_set_int_char_handlers, (cc_t *));
+extern unsigned int OS_ctty_num_int_chars (void);
+extern cc_t * OS_ctty_get_int_chars (void);
+extern cc_t * OS_ctty_get_int_char_handlers (void);
+extern void OS_ctty_set_int_chars (cc_t *);
+extern void OS_ctty_set_int_char_handlers (cc_t *);
 
 #endif /* SCM_OSCTTY_H */
