@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: crypto.scm,v 14.21 2008/01/30 20:02:29 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -302,7 +300,7 @@ USA.
 	((%md5-available?)
 	 (%md5-file filename))
 	(else
-	 (error "No MD5 support available."))))
+	 (error "This Scheme system was built without MD5 support."))))
 
 (define (%md5-file filename)
   (call-with-binary-input-file filename
@@ -332,7 +330,7 @@ USA.
 	((%md5-available?)
 	 (%md5-substring string start end))
 	(else
-	 (error "No MD5 support available."))))
+	 (error "This Scheme system was built without MD5 support."))))
 
 (define (%md5-substring string start end)
   (let ((context ((ucode-primitive md5-init 0))))
