@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: make.scm,v 4.44 2003/04/25 03:46:17 cph Exp $
-
-Copyright 1987,1988,1989,1990,1991,1994 Massachusetts Institute of Technology
-Copyright 1995,1998,1999,2000,2001,2002 Massachusetts Institute of Technology
-Copyright 2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -20,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -29,15 +27,9 @@ USA.
 
 (declare (usual-integrations))
 
-(with-working-directory-pathname
-    (directory-pathname (current-load-pathname))
+(with-loader-base-uri (system-library-uri "sf/")
   (lambda ()
-    ((access with-directory-rewriting-rule
-	     (->environment '(RUNTIME COMPILER-INFO)))
-     (working-directory-pathname)
-     (pathname-as-directory "sf")
-     (lambda ()
-       (load-package-set "sf")))
+    (load-package-set "sf")
     ((package/reference (find-package '(SCODE-OPTIMIZER))
 			'USUAL-INTEGRATIONS/CACHE!))))
 (add-subsystem-identification! "SF" '(4 41))
\ No newline at end of file
